//
//  TF_tool.swift
//  TalkfunSDKDemo
//
//  Created by 莫瑞权 on 2022/4/8.
//  Copyright © 2022 Talkfun. All rights reserved.
//

import Foundation
import UIKit
extension UIDevice {
    
    /// 顶部安全区高度
    static func xp_safeDistanceTop() -> CGFloat {
        if #available(iOS 13.0, *) {
            let scene = UIApplication.shared.connectedScenes.first
            guard let windowScene = scene as? UIWindowScene else { return 0 }
            guard let window = windowScene.windows.first else { return 0 }
            return window.safeAreaInsets.top
        } else if #available(iOS 11.0, *) {
            guard let window = UIApplication.shared.windows.first else { return 0 }
            return window.safeAreaInsets.top
        }
        return 0;
    }
    
    /// 底部安全区高度
    static func xp_safeDistanceBottom() -> CGFloat {
        if #available(iOS 13.0, *) {
            let scene = UIApplication.shared.connectedScenes.first
            guard let windowScene = scene as? UIWindowScene else { return 0 }
            guard let window = windowScene.windows.first else { return 0 }
            return window.safeAreaInsets.bottom
        } else if #available(iOS 11.0, *) {
            guard let window = UIApplication.shared.windows.first else { return 0 }
            return window.safeAreaInsets.bottom
        }
        return 0;
    }
    
    /// 顶部状态栏高度（包括安全区）
    static func xp_statusBarHeight() -> CGFloat {
        var statusBarHeight: CGFloat = 0
        if #available(iOS 13.0, *) {
            let scene = UIApplication.shared.connectedScenes.first
            guard let windowScene = scene as? UIWindowScene else { return 0 }
            guard let statusBarManager = windowScene.statusBarManager else { return 0 }
            statusBarHeight = statusBarManager.statusBarFrame.height
        } else {
            statusBarHeight = UIApplication.shared.statusBarFrame.height
        }
        return statusBarHeight
    }
    
    /// 导航栏高度
    static func xp_navigationBarHeight() -> CGFloat {
        return 44.0
    }
    
    /// 状态栏+导航栏的高度
    static func xp_navigationFullHeight() -> CGFloat {
        return UIDevice.xp_statusBarHeight() + UIDevice.xp_navigationBarHeight()
    }
    
    /// 底部导航栏高度
    static func xp_tabBarHeight() -> CGFloat {
        return 49.0
    }
    
    /// 底部导航栏高度（包括安全区）
    static func xp_tabBarFullHeight() -> CGFloat {
        return UIDevice.xp_tabBarHeight() + UIDevice.xp_safeDistanceBottom()
    }
    
}
extension UIColor {
    static func randomColor(alpha:CGFloat? = nil)-> UIColor {
        let red = CGFloat.random(in: 0...1)
        let green = CGFloat.random(in: 0...1)
        let blue = CGFloat.random(in: 0...1)
        var newAlpha:CGFloat = 1
        if(alpha != nil)
        {
            newAlpha = alpha!
        }
        return UIColor(red: red, green: green, blue: blue, alpha: newAlpha)
    }
    func colorWithAlpha(alpha: CGFloat) -> UIColor {
       var red: CGFloat = 0.0
       var green: CGFloat = 0.0
       var blue: CGFloat = 0.0
       var currentAlpha: CGFloat = 0.0

       if self.getRed(&red, green: &green, blue: &blue, alpha: &currentAlpha) {
           return UIColor(red: red, green: green, blue: blue, alpha: alpha)
       } else {
           return self
       }
   }
}
//扩展
extension NSObject {
    func currentTextView(_ textView:UIView)->Bool
    {
        if let view = UIResponder.tf_firstResponder() {
            
            if (view is UITextView){

                let firstResponder = view as! UITextView
                
                //不是聊天不用处理
                if (firstResponder == textView) {
                    return true
                }
            }else if (view is UITextField){
                
                let firstResponder = view as! UITextField
                
                //不是聊天不用处理
                if (firstResponder == textView) {
                    return true
                }
            }
            
        }
        return false
    }
    var TF_className: String {
        return NSStringFromClass(type(of: self))
    }
    /// iPhoneX、iPhoneXR、iPhoneXs、iPhoneXs Max等
      /// 判断刘海屏，返回true表示是刘海屏
      ///
      var isNotchScreen: Bool {
      
          
          if UIDevice.current.userInterfaceIdiom == .pad {
      
              return false
          }
          
          let size = UIScreen.main.bounds.size
          let notchValue: Int = Int(size.width/size.height * 100)
          
          if 216 == notchValue || 46 == notchValue {
      
              
              return true
          }
          
          return false
      }
    
    
    func TF_toast(_ msg:String)
    {

        TalkfunSharedTools.keyWindow().toast(msg, position: CGPoint(x: UIScreen.main.bounds.width/2, y: UIScreen.main.bounds.height/2))
    }
    var isNSArray_wl:Bool
    {
        
        if self.TF_className.hasPrefix("__NSFrozenDictionaryM") {
          return true
        }
        if self.TF_className.hasPrefix("__NSArray") {
          return true
        }
        if self.TF_className.contains("Array") {
             
            return true
        }
        
        
       
        return false
    }
    var isNSDictionary_wl:Bool
    {
        if self.TF_className.hasPrefix("__NSSingleEntryDictionaryI") {
            
            return true
        }
        if self.TF_className.hasPrefix("__NSDictionary") {
            
            return true
        }
        
        if self.TF_className.contains("NSDictionary") {
            
            return true
        }
        
        return false
    }

}
extension CATextLayer {
    
    @objc static func TF_watermark(text:String,font:UIFont,point:CGPoint)-> CATextLayer
    {
        
        let textLayer = CATextLayer()
        
        let myString = "计算高度专用"
        let attrsSize: CGSize = myString.size(withAttributes: [NSAttributedString.Key.font: font])
        

        var  hh:CGFloat = 0
        
        if (font.pointSize<=20) {
            hh = 5
        }
        
        textLayer.frame = CGRect(x: point.x, y: point.y, width: attrsSize.width, height: attrsSize.height+hh)
       
        textLayer.backgroundColor = UIColor.clear.cgColor
        
        // 设置文字属性
        textLayer.foregroundColor = UIColor(red: 246/255.0, green: 246/255.0, blue: 246/255.0, alpha: 1).cgColor
        textLayer.alignmentMode = CATextLayerAlignmentMode.left
        textLayer.isWrapped = false
        
      
        let fontRef = CGFont.init(font.fontName as CFString)
                textLayer.font = fontRef
                textLayer.fontSize = font.pointSize

        
        // 选择文本
        textLayer.string = text
        
        textLayer.contentsScale = UIScreen.main.scale
        textLayer.shadowOpacity = 0.8
        
        
        textLayer.shadowOffset = CGSize(width: 0, height: 1)
        textLayer.shadowRadius = 3.0
        
        return textLayer;
    }

}

extension UIFont {
    
    @objc static func PingFang(size:CGFloat)-> UIFont
    {
      var font = UIFont.init(name: "PingFangSC-Regular", size: size)
        if font == nil {
            font = UIFont.systemFont(ofSize: size)
        }
        
        return font!
    }
    @objc static func nameFang(_ name:String , size:CGFloat)-> UIFont
    {
      var font = UIFont(name: name, size: size)
        if font == nil {
            font = UIFont.systemFont(ofSize: size)
        }
        
        return font!
    }
    
}

extension UIView {
    /// 设置视图的浅色阴影和圆角
    ///
    /// - Parameters:
    ///   - cllor: 阴影的颜色
    ///   - cornerRadius: 圆角的半径
    func setLightShadow(cllor: UIColor, cornerRadius: CGFloat,backgroundColor:UIColor) {
        //一定要设置非透明的背景色才能看到阴影
        self.backgroundColor = backgroundColor
        self.layer.masksToBounds = false
        self.layer.shadowOffset = CGSize(width: 0, height: 0)
        self.layer.shadowOpacity = 1
        self.layer.shadowColor = cllor.cgColor
        self.layer.shadowRadius = 10
        
        // 设置圆角
        self.layer.cornerRadius = cornerRadius
        
        // 设置 shadowPath，确保阴影跟随圆角
        updateShadowPath()
    }
    
    /// 更新阴影路径以匹配圆角
    private func updateShadowPath() {
        let cornerRadius = self.layer.cornerRadius
        let shadowPath = UIBezierPath(roundedRect: self.bounds, cornerRadius: cornerRadius)
        self.layer.shadowPath = shadowPath.cgPath
    }
    
    /// 在视图布局发生变化时更新阴影路径
    func updateShadowPathIfNeeded() {
        if self.layer.shadowPath != nil {
            updateShadowPath()
        }
    }
    func roundedCorners(rect: CGRect?=nil,corners: UIRectCorner,cornerRadii: CGSize,borderWidth: CGFloat?=nil,borderColor:UIColor?=nil)
    {
        
        self.layer.mask = nil
        var newRect = CGRect(x: 0, y: 0, width: self.frame.size.width, height: self.frame.size.height)
        if(rect != nil)
        {
            newRect = rect!
        }
      
        let maskPath_2 = UIBezierPath.init(roundedRect: newRect, byRoundingCorners: corners, cornerRadii: cornerRadii)
         
        let maskLayer_2 = CAShapeLayer()
        maskLayer_2.frame = CGRect(x: 0, y: 0, width: self.frame.size.width, height: self.frame.size.height)
        maskLayer_2.path = maskPath_2.cgPath
         
        self.layer.mask = maskLayer_2
        
        if let borderWidth = borderWidth ,let borderColor = borderColor
        {
            // 创建绘制边框的图层
              let borderLayer = CAShapeLayer()
              borderLayer.path = maskPath_2.cgPath // 使用与mask相同的路径
              borderLayer.frame = self.bounds
              borderLayer.lineWidth = borderWidth
              borderLayer.strokeColor = borderColor.cgColor
              borderLayer.fillColor = UIColor.clear.cgColor // 确保填充色透明，这样不会遮盖视图的内容
              borderLayer.frame = CGRect(x: 0, y: 0, width: self.frame.size.width, height: self.frame.size.height)

              // 如果已经有边框图层，先移除
              if let sublayers = self.layer.sublayers {
                  for layer in sublayers where layer is CAShapeLayer {
                      layer.removeFromSuperlayer()
                  }
              }
              
              self.layer.addSublayer(borderLayer)
        }

        
    }
    func setLightGrayShadow() {
        //灰色阴影
         self.layer.shadowOffset = CGSize(width: 1, height: 1)
         self.layer.shadowOpacity = 0.5
         self.layer.shadowColor = UIColor.lightGray.cgColor
    }
    func tf_roundedCorners(rect: CGRect?=nil,corners: UIRectCorner,cornerRadii: CGSize)
     {
         
         self.layer.mask = nil
         var newRect = self.bounds
         if(rect != nil)
         {
             newRect = rect!
         }
       
         let maskPath_2 = UIBezierPath.init(roundedRect: newRect, byRoundingCorners: corners, cornerRadii: cornerRadii)
          
         let maskLayer_2 = CAShapeLayer()
         maskLayer_2.frame = self.bounds
         maskLayer_2.path = maskPath_2.cgPath
          
         self.layer.mask = maskLayer_2
         
     }
    func maskPath_new( size: CGSize)
    {
        let maskPath = UIBezierPath.init(roundedRect: self.bounds, byRoundingCorners:[UIRectCorner.topLeft,UIRectCorner.topRight], cornerRadii: size)
       
        let maskLayer = CAShapeLayer()
        maskLayer.frame = self.bounds
        maskLayer.path = maskPath.cgPath
        self.layer.mask = maskLayer
    }
    //单击的手势
    @objc func clickGesture(target: Any? ,action: Selector?)->UITapGestureRecognizer
    {
        let singleRecognizer = UITapGestureRecognizer.init(target: target, action: action)
        /*单击    -------    点击的次数*/
            singleRecognizer.numberOfTapsRequired = 1
        self.addGestureRecognizer(singleRecognizer)


        return singleRecognizer
    }
    //双击的手势
    @objc func doubleClickGesture(target: Any? , action: Selector?)->UITapGestureRecognizer
    {
        
        let singleRecognizer = UITapGestureRecognizer.init(target: target, action: action)
        /*双击    -------    点击的次数*/
            singleRecognizer.numberOfTapsRequired = 2
        self.addGestureRecognizer(singleRecognizer)
        return singleRecognizer
    }
    

}

extension NSMutableAttributedString {
    func setAlignment(_ talignment:NSTextAlignment)->NSMutableAttributedString
    {
        // 创建一个段落样式
        let paragraphStyle = NSMutableParagraphStyle()
        paragraphStyle.alignment = talignment // 设置文本居中对齐

        // 将段落样式应用于NSMutableAttributedString
        self.addAttribute(NSAttributedString.Key.paragraphStyle, value: paragraphStyle, range: NSRange(location: 0, length: self.length))
        
        return self
    }
    @objc static func getAttributed(string: String, font: UIFont, foregroundColor: UIColor, shadow: NSShadow? = nil) -> NSMutableAttributedString {
        
        var attributes: [NSAttributedString.Key: Any] = [
            .backgroundColor: UIColor.clear,
            .font: font,
            .foregroundColor: foregroundColor
        ]
        
        // 如果提供了shadow属性，则添加到属性字典中
        if let shadow = shadow {
            attributes[.shadow] = shadow
        }
        
        return NSMutableAttributedString(string: string, attributes: attributes)
    }
    @objc func getWidth() ->CGFloat
    {
        let rect = self.boundingRect(with: CGSize(width: CGFloat.greatestFiniteMagnitude, height: CGFloat.greatestFiniteMagnitude), options: [.usesLineFragmentOrigin], context: nil)
        return rect.width
    }
    //
    @objc func attributes(url:String,font:UIFont ,color:UIColor,link:String)
    {

        let attr35: [NSAttributedString.Key : Any] = [.font: font,.foregroundColor: color,]
        
        let str1:NSString  = self.string as NSString
        let range = str1.range(of: url as String)
     
        self.addAttributes(attr35, range: range)
  
        let attr36: [NSAttributedString.Key : Any] = [.underlineStyle: 1,]
        self.addAttributes(attr36, range: range)

        let attr37: [NSAttributedString.Key : Any] = [.link: link]
        self.addAttributes(attr37, range: range)
        
    }
        
    @objc static func completeStr(_ completeStr:String,splitArray:NSArray)->NSMutableArray
    
    {
        
        //所有表情的位置与内容
        let subStrArray = NSMutableArray()
        for i in 0..<splitArray.count {

            let name = splitArray[i] as! String
            //单位表情的位置与内容
            
            let subStr  =  self.getDuplicateSubStrLocInCompleteStr(completeStr, subStr: name)
            subStrArray.add(subStr)
         
        }
        
        
        
        
        let indexArray  = NSMutableArray()
        for i in 0..<subStrArray.count {

            let dict = subStrArray[i] as! NSDictionary
            
            let index = dict["index"] as! NSNumber
            indexArray.add(index)
          
        }
        
        
        //从小到大排序

        let  sortedArr = self.selectionSort(array: indexArray as! [Int])
        
        let new = NSMutableArray()
        //字典排序

        for i in 0..<sortedArr.count {
        
            let index = sortedArr[i] as NSNumber
            
            for i in 0..<subStrArray.count {
           
                let dict = subStrArray[i] as!NSDictionary
                let index2 = dict["index"] as! NSNumber
               
                if index.intValue == index2.intValue {
                   new.add(dict)
               }
                
            }
             
           
        }
        
        return new
    }
    // MARK: - 快排
    // 输入一个数组，返回从小到大排序的数组

    static func selectionSort(array: [Int]) -> [Int] {
        var sortArray = array
        for i in 0..<sortArray.count {
            var minValueIndex = i
            for j in (minValueIndex + 1)..<sortArray.count {
                if sortArray[minValueIndex] > sortArray[j] { // 得到最小值位置
                    minValueIndex = j
                }
            }
            // 与最小值进行交换
            let temp = sortArray[i]
            sortArray[i] = sortArray[minValueIndex]
            sortArray[minValueIndex] = temp
        }
        return sortArray
    }
    //利用切分先得数组,再根据索引计算
    @objc static func getDuplicateSubStrLocInCompleteStr(_ completeStr:String,subStr:String)->NSMutableArray
   
    {
        
        let separatedStrArr = completeStr.components(separatedBy: subStr)
        let locMuArr = NSMutableArray()
        
        var index:NSInteger = 0
        for i in 0..<separatedStrArr.count-1 {

            let separatedStr = separatedStrArr[i] as String
            index = index + separatedStr.count
           
            let loc_num =  NSNumber.init(value: index)
            
            //表情所在位置与表情
            
            let dict = NSMutableDictionary()
            dict.setObject(loc_num, forKey: "index" as NSCopying)
            dict.setObject(subStr, forKey: "subStr" as NSCopying)

            locMuArr.add(dict)
         
            index = index+subStr.count
        }
        return locMuArr;
    }
    
    
     //111111
     static func getArray()->NSMutableArray
    {
        let array =  ["[hard]","[aha]","[why]","[Fireworks]","[pitiful]","[amaz]","[good]","[cool]","[bye]","[rose]","[slime]","[laugh]","[love]","[tongue]","[daze]","[handclap]","[titter]","[handssors]","[six]","[heart]","[melon]","[silly]","[dog]","[like]","[expect]","[666]","[flower]"]
        
   
        return NSMutableArray.init(array: array)
    }
    static func getExpressionDict()->NSMutableDictionary

    {

//        let array:NSMutableArray = self.getArray()
//        for i in 0..<35 {
//
//            if i>15 {
//                array.add(String.init(format: "[emoticon%i]", i+1))
//            }
//
//
//        }
        
        let expressionDict = NSMutableDictionary()
        expressionDict.setObject("[hard]", forKey: "[fd]" as NSCopying)
        expressionDict.setObject("[aha]", forKey: "[lg]" as NSCopying)
        expressionDict.setObject("[why]", forKey: "[qu]" as NSCopying)
        expressionDict.setObject("[Fireworks]", forKey: "[fl]" as NSCopying)
        expressionDict.setObject("[pitiful]", forKey: "[kl]" as NSCopying)
        expressionDict.setObject("[amaz]", forKey: "[jy]" as NSCopying)
        
        expressionDict.setObject("[good]", forKey: "[[gz]]" as NSCopying)
        expressionDict.setObject("[cool]", forKey: "[hx]" as NSCopying)
        expressionDict.setObject("[love]", forKey: "[tx]" as NSCopying)
        expressionDict.setObject("[bye]", forKey: "[ag]" as NSCopying)
        expressionDict.setObject("rose", forKey: "[S_FLOWER]" as NSCopying)
        
        return expressionDict;
    }
    
    static func getBigExpressionDict()->NSDictionary
    {
        return ["[slime]":"[slime]",
                 "[laugh]":"[laugh]",
                 "[love]":"[love]",
                 "[tongue]":"[tongue]",
                 "[daze]":"[daze]",
                 "[handclap]":"[handclap]",
                 "[titter]":"[titter]",
                 "[handssors]":"[handssors]",
                 "[six]":"[six]",
                 "[heart]":"[heart]",
                 "[melon]":"[melon]",
                 "[silly]":"[silly]",
                 "[dog]":"[dog]",
                 "[like]":"[like]",
                 "[expect]":"[expect]",
                 "[flower]":"[flower]",
                 "[色]":"[emoticon17]",
                 "[调皮]":"[emoticon18]",
                 "[害羞]":"[emoticon19]",
                 "[奸笑]":"[emoticon20]",
                 "[尴尬]":"[emoticon21]",
                 "[惊讶]":"[emoticon22]",
                 "[闭嘴]":"[emoticon23]",
                 "[呲牙]":"[emoticon24]",
                 "[大哭]":"[emoticon25]",
                
                 "[囧]":"[emoticon26]",
                 "[难过]":"[emoticon27]",
                 "[敲打]":"[emoticon28]",
                 "[委屈]":"[emoticon29]",
                 "[疑问]":"[emoticon30]",
                 "[郁闷]":"[emoticon31]",
                 "[再见]":"[emoticon32]",
                 "[可怜]":"[emoticon33]",
                 "[捂脸]":"[emoticon34]",
                 "[笑哭]":"[emoticon35]",
                 "[安慰]":"[emoticon26]",
                 "[666]":"[six]",
    ]
    }
    //生成图文富文本
//    static func getAttributedText(_ str:String,font:UIFont,textColor:UIColor)->NSMutableAttributedString
////    + (NSMutableAttributedString*)getAttributedText:(NSString *)str fontSize:(UIFont*)font foregroundColor:(UIColor*)color
//    {
//
//        let expressionDict = self.getExpressionDict()
//        let expressionDict2 = self.getBigExpressionDict()
//        
//        for i in 0..<expressionDict2.count {
//
//        //  获取字典中所有的key
//            
//            
//         let keys = expressionDict2.allKeys
//        //  取出当前位置对应的key
//         
//         let key = keys[i]
//         let value = expressionDict2[key]
//                     
//            expressionDict.setObject(value as Any, forKey: key as! NSCopying)
//            
////            [expressionDict setObject:value forKey:key];
//
//         }
//        
//        var newStr = str as NSString
//            //将字符串中" name" 全部替换成 expressionDict[name]
//        
//        //例如:找到聊天内容的 "[fd]"换成 @"[hard]"
//        
//        for i in 0..<expressionDict.count {
//            
//            let name = expressionDict[i] as! String
//            newStr = newStr.replacingOccurrences(of:name, with:expressionDict[name] as! String ) as NSString
//
//        }
//        
//       
//        //1.图文混排
//        let String = NSMutableAttributedString()
//        
//        //{
//    //        "index" : 0,
//    //        "subStr" : "[aha]"
//    //    }
//       
//        let subStrArray =  self.completeStr(str, splitArray: self.getArray())
//    
//        let splitter = "(欢拓分割符号)"
//        
//        //占位替换
//        for i in 0..<subStrArray.count {
//         let dict = subStrArray[i] as! NSDictionary
//    
//            //替换 "[aha]" 为 @"(欢拓分割符号)"
//            newStr = newStr.replacingOccurrences(of:dict["subStr"] as! String, with:splitter ) as NSString
//            
//        }
//
//        
//        //分割数据
//        
//        let nameArray = newStr.components(separatedBy: splitter)
//        
//        //找出所有字符串
//        for i in 0..<nameArray.count
//     
//        {
//            
//            let key =  nameArray[i] as String
//           
//            if (key != nil)
//            {
//                
//                
//                //拿到整体的字符串
//                
//                let name = NSMutableAttributedString.init(string: key)
//                
//                // 设置 name 颜色，也可以设置字体大小等
//                
//                [name addAttribute:NSForegroundColorAttributeName value:color range:NSMakeRange(0, name.length)];
//                
//                name.addAttributes(range: <#T##NSRange#>)
//                
//                
//                
//                [name addAttribute:NSFontAttributeName value:font range:NSMakeRange(0, name.length)];
//                
//                [String appendAttributedString:name];
//                
//                
//                if (i < subStrArray.count) {
//                    NSDictionary *dict =    subStrArray[i];
//                    
//                    // 创建图片图片附件
//                    NSTextAttachment *attach = [[NSTextAttachment alloc] init];
//                    
//                    NSString*name5 = dict[@"subStr"];
//                    
//                    //替换 "[3]" 为 @"(欢拓分割符号)"
//                    name5 = [name5 stringByReplacingOccurrencesOfString :@"[" withString:@""];
//                    
//                    name5 = [name5 stringByReplacingOccurrencesOfString :@"]" withString:@""];
//                    
//                    
//                    attach.image = [UIImage imageNamed:name5];
//                    attach.bounds = CGRectMake(0, -5 , 22, 22);
//                    
//                    NSAttributedString *attachString = [NSAttributedString attributedStringWithAttachment:attach];
//                    
//                    //将图片插入到合适的位置
//                    [String insertAttributedString:attachString atIndex:String.length];
//                }
//                
//                
//                
//                
//            }
//            
//        }
//        
//        
//        return String;
//        
//    }

}

extension String {
    
    func calculate_the_height(labelWidth:CGFloat,font:UIFont ) ->CGFloat
    {
       
       let label = UILabel.init(frame:CGRect(x: 0, y: 0, width: labelWidth, height: 1.0))
        
        label.font = font;
        label.numberOfLines = 0;
        label.attributedText = NSMutableAttributedString.init(string: self);
        label.lineBreakMode = NSLineBreakMode.byCharWrapping;


        let size = label.sizeThatFits(CGSize(width: label.frame.size.width, height: CGFloat.greatestFiniteMagnitude))
       
        return size.height
    }

}
extension Int {
    func ToString()->String
    {
        return  NumberConversion.ToString(CGFloat(self))
    }
//    func kOrM()->String
//    
//    {
//        return  NumberConversion.kOrM(CGFloat(self))
//    }
}
extension NSInteger {
    func ToString2()->String
    {
        return  NumberConversion.ToString(CGFloat(self))
    }
//    func kOrM2()->String
//    
//    {
//        return  NumberConversion.kOrM(CGFloat(self))
//    }
}
class NumberConversion: NSObject {
    
    static func ToString(_ number:CGFloat)->String
    {
        
        let formatter = NumberFormatter()
        formatter.maximumFractionDigits = 10 // 设置小数点后的最大位数
        formatter.minimumFractionDigits = 0 // 设置小数点后的最小位数
        let string = formatter.string(from: NSNumber(value: Double(number))) ?? ""
        
        
        return  string
    }
    
}
extension UIViewController {
    
    func getCalculateViewFrame(_ isOrientationLandscape:Bool)->CGRect
   
    {
        
        let width = UIScreen.main.bounds.size.width
        let height = UIScreen.main.bounds.size.height
        
        //横屏
        if (isOrientationLandscape) {
            if (width<height) {
                let new_width = UIScreen.main.bounds.size.height
                let new_height =  UIScreen.main.bounds.size.width
                 return CGRect(x: 0, y: 0, width: new_width, height: new_height)
              
            }
           
        }else
        {
            if (width>height) {
                let new_width = UIScreen.main.bounds.size.height
                let new_height =  UIScreen.main.bounds.size.width
                 return CGRect(x: 0, y: 0, width: new_width, height: new_height)
            }
        }
        return CGRect(x: 0, y: 0, width: width, height: height)
       
    }
    
    func TF_dismissUI()
    {
        TalkfunSharedTools.removeFromParentViewController(self, name: "TalkfunScanNewLoginViewController")
        TalkfunSharedTools.removeFromParentViewController(self, name: "TalkfunScanViewController")
        TalkfunSharedTools.removeFromParentViewController(self, name: "TalkfunSecondNewLoginViewController")

         //返回上一页
         if (self.navigationController != nil) {
             self.navigationController?.popViewController(animated: true)

         }else{
             self.dismiss(animated: true)

         }
    }
    
    func TF_alertController(_ msg:String)
    {
        let alert:UIAlertController = UIAlertController.init(title: "温馨提示", message: msg, preferredStyle: .alert)
        let otherAction:UIAlertAction = UIAlertAction.init(title: "确认", style: .default) { action in
            
        }
        alert.addAction(otherAction)
        self.present(alert, animated: true, completion: nil)
    }
    /** 获取当前控制器 */
    static func TF_current() -> UIViewController {
        let keyWindow = TalkfunSharedTools.keyWindow()
        let vc = keyWindow.rootViewController
        return UIViewController.TF_findBest(vc: vc!)
    }
    
    private static func TF_findBest(vc: UIViewController) -> UIViewController {
        if vc.presentedViewController != nil {
            return UIViewController.TF_findBest(vc: vc.presentedViewController!)
        } else if vc.isKind(of: UISplitViewController.self) {
            let svc = vc as! UISplitViewController
            if svc.viewControllers.count > 0 {
                return UIViewController.TF_findBest(vc: svc.viewControllers.last!)
            } else {
                return vc
            }
        } else if vc.isKind(of: UINavigationController.self) {
            let svc = vc as! UINavigationController
            if svc.viewControllers.count > 0 {
                return UIViewController.TF_findBest(vc: svc.topViewController!)
            } else {
                return vc
            }
        } else if vc.isKind(of: UITabBarController.self) {
            let svc = vc as! UITabBarController
            if (svc.viewControllers?.count ?? 0) > 0 {
                return UIViewController.TF_findBest(vc: svc.selectedViewController!)
            } else {
                return vc
            }
        } else {
            return vc
        }
    }
}
extension NSAttributedString {
 
   @objc static func getAttributed(string:String,font:UIFont,foregroundColor:UIColor)-> NSMutableAttributedString {
        
        let attributes = [NSMutableAttributedString.Key.backgroundColor:UIColor.clear, NSMutableAttributedString.Key.font:font as Any,NSMutableAttributedString.Key.foregroundColor: foregroundColor]
        

       let attributedString = NSMutableAttributedString.init(string:string as String, attributes: attributes)

     
        return attributedString

    }
    
     /**计算单行富文本字符串的高度。*/
    @objc func getHeight() -> CGFloat {
           let textStorage = NSTextStorage(attributedString: self)
           let textContainer = NSTextContainer(size: CGSize(width: CGFloat.greatestFiniteMagnitude, height: CGFloat.greatestFiniteMagnitude))
           let layoutManager = NSLayoutManager()

           layoutManager.addTextContainer(textContainer)
           textStorage.addLayoutManager(layoutManager)

           textContainer.lineBreakMode = .byWordWrapping
           layoutManager.glyphRange(for: textContainer)
           
           let rect = layoutManager.usedRect(for: textContainer)
           return ceil(rect.size.height)
       }
    
    func requiredHeight(forWidth width: CGFloat) -> CGFloat {
            // 定义一个具有最大宽度和无限高度的CGSize
            let size = CGSize(width: width, height: CGFloat.greatestFiniteMagnitude)
            
            // 使用boundingRect计算给定宽度下的字符串的边界矩形
            let options: NSStringDrawingOptions = [.usesLineFragmentOrigin, .usesFontLeading]
            let boundingRect = self.boundingRect(with: size, options: options, context: nil)
            
            // 返回所需的高度
            return ceil(boundingRect.height)
    }
}

import ObjectiveC
private var ActionBlockKey: UInt8 = 0
typealias UIButtonActionClosure = () -> Void


class homeTagViewButton: UIButton {

    var name:String = ""

}
extension UIButton {
    func setCustomFont(_ font: UIFont) {
        guard let titleLabel = self.titleLabel else {
            print("Error: titleLabel is nil")
            return
        }
        titleLabel.font = font
    }
    
    func getCustomFont()->UIFont
    {
        guard let titleLabel = self.titleLabel else {
            print("Error: titleLabel is nil")
            return UIFont.PingFang(size: 14)
        }
        
        return  titleLabel.font
    }
    
    //------------------------------
    private class ActionClosureWrapper {
        let closure: UIButtonActionClosure
        
        init(_ closure: @escaping UIButtonActionClosure) {
            self.closure = closure
        }
    }
    
    func addAction(for controlEvents: UIControl.Event, action: @escaping UIButtonActionClosure) {
        objc_setAssociatedObject(self, &ActionBlockKey, ActionClosureWrapper(action), .OBJC_ASSOCIATION_RETAIN_NONATOMIC)
        addTarget(self, action: #selector(handleAction), for: controlEvents)
    }
    
    @objc private func handleAction() {
        if let wrapper = objc_getAssociatedObject(self, &ActionBlockKey) as? ActionClosureWrapper {
            wrapper.closure()
        }
    }
}
extension NSDictionary {
    func convertDictionaryToString() -> String {
        //dict:[String:AnyObject]
      var result:String = ""
      do {
        //如果设置options为JSONSerialization.WritingOptions.prettyPrinted，则打印格式更好阅读
        let jsonData = try JSONSerialization.data(withJSONObject: self, options: JSONSerialization.WritingOptions.init(rawValue: 0))

        if let JSONString = String(data: jsonData, encoding: String.Encoding.utf8) {
          result = JSONString
        }

      } catch {
        result = ""
      }
      return result
    }
    
    func getString_forKey(_ key:String)->String
    {
      
        if self[key] != nil {
            let str = self.getNumber(self[key] as! NSObject)
            return str

        }
        
        return ""
    }
 
    func getBool_forKey(_ key:String)->Bool
    {
        let numbe = self.getInt_forKey(key)
        if(numbe==0)
        {
            return false
        }
        
        return true
    }
    func getInt_forKey(_ key:String)->Int
    {
        var number2 = 0
        if self[key] != nil {
            let str = self.getString_forKey(key)

            if (str.count>0)
            {

                
                let formatter = NumberFormatter()
                formatter.numberStyle = .decimal
                if let number = formatter.number(from: str) {
                    
                    number2 = number.intValue
               
                }
            }
            
        }
        
        return number2
    }
    func getInt64_forKey(_ key:String)->Int64
    {
        var number2:Int64 = 0
        if self[key] != nil {
            let str = self.getString_forKey(key)
            if (str.count>0)
            {

                let formatter = NumberFormatter()
                formatter.numberStyle = .decimal
                if let number = formatter.number(from: str) {
                    
                    number2 = number.int64Value
               
                }
            }
        }
        
        return number2
    }
    func getInteger_forKey(_ key:String)->NSInteger
    {
        var number2 = 0
        if self[key] != nil {
            let str = self.getString_forKey(key)
            if (str.count>0)
            {

                let formatter = NumberFormatter()
                formatter.numberStyle = .decimal
                if let number = formatter.number(from: str) {
                    
                    number2 = number.intValue
               
                }
            }
        }
        
        return number2
    }
    func getFloat_forKey(_ key:String)->CGFloat
    {
        var cgFloatValue:CGFloat = 0
        if self[key] != nil {
            let str = self.getString_forKey(key)
            
            if (str.count>0)
            {
                
                cgFloatValue = self.getFloat(from: self.getNumber_0(self[key] as! NSObject))
                
            }
        }
        
        return cgFloatValue
    }
    func getDict_forKey(_ key:String)->NSMutableDictionary
    
    {
        let dict2 = NSMutableDictionary()
        if self[key] != nil {
            let obj = self[key] as! NSObject
            if(obj.isNSDictionary_wl)
            {
                dict2.addEntries(from: (self[key] as! NSDictionary) as! [AnyHashable : Any])
            }
            
        }
        
        return dict2
    }
    func getArray_forKey(_ key:String)->NSMutableArray
    
    {
        let new_array = NSMutableArray()
        if self[key] != nil {
            let new_obj = self[key] as! NSObject
            if new_obj.isNSArray_wl {
                let array = self[key] as! NSArray
                new_array.addObjects(from: array as! [Any])
              
            }
            
        }
        
        return new_array
    }
    @objc func getNumber(_ obj:NSObject)->String
    {
        var UsersXid:String = ""
        
        if(obj is NSNumber){

            let num:NSNumber = obj as!NSNumber
   
            UsersXid = NSString.init(format: "%i", num) as String
        }

        return UsersXid;
    }
    @objc func getNumber_0(_ obj:NSObject)->String
    {
        var UsersXid:String = ""
        
        if(obj is NSNumber){

            let num:NSNumber = obj as!NSNumber
           
            
            UsersXid = NSString.init(format: "%i", num) as String
        }

        return UsersXid;
    }
     func getFloat(from string: String) -> CGFloat {
          return CGFloat((string as NSString).doubleValue)
      }
    
    

    
}
