//
//  TFtouchesMoved.swift
//  TalkfunSDKDemo
//
//  Created by 莫瑞权 on 2022/4/28.
//  Copyright © 2022 Talkfun. All rights reserved.
//

import UIKit
// 定义一个协议来复用触摸移动的逻辑
protocol Movable {
    func handleTouchesMoved(_ touches: Set<UITouch>)
}

extension Movable where Self: UIView {
  
    func handleTouchesMoved(_ touches: Set<UITouch>) {
        let touch = touches.first!
        // 获取当前点
        let secondPoint = touch.location(in: self)
        // 获取上一个点
        let firstPoint = touch.previousLocation(in: self)
        // 计算两点间的距离
        let offsetX: CGFloat = secondPoint.x - firstPoint.x
        let offsetY: CGFloat = secondPoint.y - firstPoint.y
        // 移动当前view
        if let superview = self.superview {
            if self.frame.origin.x + offsetX > 0 &&
               self.frame.origin.y + offsetY > 0 &&
               self.frame.origin.y + offsetY < superview.frame.size.height - self.frame.size.height &&
               self.frame.origin.x + offsetX < superview.frame.size.width - self.frame.size.width {

                self.frame = CGRect(x: self.frame.origin.x + offsetX,
                                    y: self.frame.origin.y + offsetY,
                                    width: self.frame.size.width,
                                    height: self.frame.size.height)
            }
        }
    }
    
    func setupShadow() {
        // 设置左边的圆角
        let cornerRadius = self.bounds.height / 2
        let roundedRect = CGRect(x: 0, y: 0, width: self.bounds.width, height: self.bounds.height)
        let path = UIBezierPath(roundedRect: roundedRect,
                                byRoundingCorners: [.topLeft, .bottomLeft],
                                cornerRadii: CGSize(width: cornerRadius, height: cornerRadius))

        // 为视图的 layer 设置阴影
        self.layer.shadowOffset = CGSize(width: 1, height: 1)
        self.layer.shadowOpacity = 0.6
        self.layer.shadowColor = UIColor.lightGray.cgColor
        self.layer.shadowPath = path.cgPath

        // 使用遮罩路径设置圆角
        let maskLayer = CAShapeLayer()
        maskLayer.path = path.cgPath
        self.layer.mask = maskLayer
    }



}

// 让 TFtouchesMoved 遵守 Movable 协议
class TFtouchesMoved: UIView, Movable {
    var hitCount:NSInteger = 0
    
    @objc public var touchesBeganBlock: TFHorizontalChatViewClickBlock?
    

    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        hitCount = hitCount+1
        handleTouchesMoved(touches)
    }
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        hitCount = 1
    }
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        
            if hitCount==1 {
                if touchesBeganBlock != nil{
                    touchesBeganBlock!()
                }
            }
       
        hitCount = 0
    }
    
    override func touchesCancelled(_ touches: Set<UITouch>, with event: UIEvent?) {
        if hitCount==1 {
            if touchesBeganBlock != nil{
                touchesBeganBlock!()
            }
        }
        hitCount = 0
    }
}

// 让 TFtouchesMovedBtn 遵守 Movable 协议
class TFtouchesMovedBtn: UIButton, Movable {
    var hitCount:NSInteger = 0
    @objc public var touchesBeganBlock: TFHorizontalChatViewClickBlock?
    override func touchesMoved(_ touches: Set<UITouch>, with event: UIEvent?) {
        hitCount = hitCount+1
        handleTouchesMoved(touches)
    }
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        hitCount = 1
    }
    override func touchesEnded(_ touches: Set<UITouch>, with event: UIEvent?) {
        
            if hitCount<6 {
                if touchesBeganBlock != nil{
                    touchesBeganBlock!()
                }
            }
       
        hitCount = 0
        if let superview = self.superview {
            self.frame = CGRect(x: superview.frame.width-self.frame.size.width ,
                                y: self.frame.origin.y,
                                width: self.frame.size.width,
                                height: self.frame.size.height)
            
        }
        
    }
    
    override func touchesCancelled(_ touches: Set<UITouch>, with event: UIEvent?) {
        if hitCount<6 {
            if touchesBeganBlock != nil{
                touchesBeganBlock!()
            }
        }
        hitCount = 0
        if let superview = self.superview {
            self.frame = CGRect(x: superview.frame.width-self.frame.size.width ,
                                y: self.frame.origin.y,
                                width: self.frame.size.width,
                                height: self.frame.size.height)
            
        }
    }
}
