//
//  TF_LuckyBagView.swift
//  TalkfunSDKDemo
//
//  Created by moRui on 2024/11/7.
//  Copyright © 2024 Talkfun. All rights reserved.
//

import UIKit
typealias countdownTimerEndsBlock = () -> Void
class TF_LuckyBagView: UIButton {
    //裤袋倒计时结束 
//    @objc public var timerEndsBlock: countdownTimerEndsBlock?
    private let background_imageView = UIImageView()
    let countdownLabel = UILabel()
    //是不是默认
    var isDefaultFrame: Bool = true
    private var countdownSeconds: NSInteger = 0
    private var countdownTimer: Timer? = nil
    
    //当前福袋id
    var currentID:NSInteger = 0
//    //上一个福袋id
//    var lastID:NSInteger = 0
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.addSubview(background_imageView)
        background_imageView.image = UIImage(named: "福袋-大")
        
        self.addSubview(countdownLabel)
        countdownLabel.isHidden = true
        countdownLabel.layer.masksToBounds = true
    }
    //结束定时器
    func endTimer()
    {
        if let timer = self.countdownTimer {
            timer.invalidate()
            self.countdownTimer = nil
        }
    }
    //TODO: 开始倒计时
    func startCountdown(_ seconds: NSInteger) {
        self.countdownSeconds = seconds
        self.countdownLabel.isHidden = false
        
        if countdownTimer == nil {
            countdownTimer = Timer.scheduledTimer(
                timeInterval: 1,
                target: self,
                selector: #selector(updateCountdown),
                userInfo: nil,
                repeats: true
            )
            RunLoop.main.add(countdownTimer!, forMode: .default)
            countdownTimer?.fire()
        }
    }
    
    @objc private func updateCountdown() {
        if countdownSeconds >= 0 {
            countdownLabel.attributedText = NSAttributedString
                .getAttributed(
                    string: formatCountdown(countdownSeconds),
                    font: UIFont.PingFang(size: 12),
                    foregroundColor: .white
                )
                .setAlignment(.center)
            countdownLabel.backgroundColor = UIColor(red: 1.0, green: 48/255, blue: 48/255, alpha: 0.8)
            
            
            NotificationCenter.default.post(name: Notification.Name(rawValue: "TF_luckyBag_details_countdown"), object:countdownSeconds.ToString() )
            
            countdownSeconds -= 1
        } else {
            self.endTimer()
            if self.superview != nil {
                self.removeFromSuperview()
            }
//            lastID = 0
//            if timerEndsBlock != nil {
//                timerEndsBlock!()
//            }
            
        }
    }
    
    private func formatCountdown(_ seconds: NSInteger) -> String {
        let minutes = seconds / 60
        let remainingSeconds = seconds % 60
        return String(format: "%02d:%02d", minutes, remainingSeconds)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        
        let imageX: CGFloat = isDefaultFrame ? 0.0 : 2.0
        let imageWidth: CGFloat = isDefaultFrame ? self.frame.width : self.frame.width - 2 * imageX
        
        background_imageView.frame = CGRect(x: imageX, y: 0, width: imageWidth, height: self.frame.height)
        countdownLabel.frame = CGRect(x: 0, y: self.frame.height - 16, width: self.frame.width, height: 16)
        countdownLabel.layer.cornerRadius = 8
    }
    
    // 横向动画，福袋从左移入
    private func animateLuckyBagHorizontalMove(initialFrame: CGRect,
                                               pageViewMode:NSInteger,
                                               completion: @escaping () -> Void) {
        UIView.animate(withDuration: 0.5, animations: {
            let luckyBagX = pageViewMode == 1 ? 18.0 : 10.0
            self.frame = CGRect(x: luckyBagX, y: initialFrame.origin.y, width: initialFrame.width, height: initialFrame.height)
        }, completion: { _ in
            completion()
        })
    }

    // 纵向动画，福袋上移到指定位置
    private func animateLuckyBagVerticalMove(finalFrame: CGRect, completion: @escaping () -> Void) {
        UIView.animate(withDuration: 0.5, animations: {
            self.isDefaultFrame = false
            self.frame = finalFrame
        }, completion: { _ in
            completion()
        })
    }

    static func end_luckyBag_frame(_ pageViewMode:NSInteger)->CGRect
    {
        return CGRect(x: 12, y: default_luckyBag_frame(pageViewMode).origin.y-44-16, width: 44, height: 44)
    }
    static func default_luckyBag_frame(_ pageViewMode:NSInteger)->CGRect
    {
        var luckyBag_y = UIDevice.xp_safeDistanceTop()+72.0
        /**pageViewMode 生活直播专用
         0二分屏  横屏
         1竖屏
         2课件模式
         3三分屏
         **/
        if pageViewMode==1 {
            luckyBag_y = UIDevice.xp_safeDistanceTop()+10+45+110
        }
        let luckyBag_width = 64.0
        return CGRect(x: -luckyBag_width, y: luckyBag_y, width: luckyBag_width, height: luckyBag_width)
    }
    //TODO: 福袋推送
    func luckyBagPush(superView:UIView,
                      pageViewMode:NSInteger,
                      countdown:NSInteger)
                             
    {
        superView.addSubview(self)
    
        let initialFrame = TF_LuckyBagView.default_luckyBag_frame(pageViewMode)
        let finalFrame = TF_LuckyBagView.end_luckyBag_frame(pageViewMode)
        
        self.frame = initialFrame
        self.isDefaultFrame = true
        self.countdownLabel.isHidden = true
        
        // 先执行横向动画
        animateLuckyBagHorizontalMove(initialFrame: initialFrame,pageViewMode: pageViewMode) {
            // 延时后再执行纵向动画
            DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
                self.animateLuckyBagVerticalMove(finalFrame: finalFrame) {
                    //TODO: 开始倒计时
                    self.startCountdown(countdown)
                }
            }
        }
        
        
    }
    //TODO: 福袋取消
    func luckyBagCance(_ model:TFLuckyBagCance )
    {
        countdownTimer?.invalidate()
        countdownTimer = nil
        
        self.removeFromSuperview()
    }
  
}
